//+------------------------------------------------------------------+
//|                                   EvenBetterSineWave_v1 600+.mq4 |
//|                                Copyright  2014, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
// List of Prices:
// Price    = 0 - Close  
// Price    = 1 - Open  
// Price    = 2 - High  
// Price    = 3 - Low  
// Price    = 4 - Median Price   = (High+Low)/2  
// Price    = 5 - Typical Price  = (High+Low+Close)/3  
// Price    = 6 - Weighted Close = (High+Low+Close*2)/4

#property copyright "Copyright  2014, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 clrRoyalBlue
#property indicator_width1  2

#property indicator_level1  0.0
#property indicator_level2  0.8
#property indicator_level3 -0.8

#define pi 3.14159265358979323846

//---- indicator parameters

extern int     TimeFrame               =     0;       //TimeFrame in min
extern int     Price                   =     0;       //Price = 0...6
extern int     Duration                =    40;       //Expected maximum duration of the trades when the market is in a trend
extern int     Smooth                  =    10;       //Super Smoother period

//---- indicator buffers
double wave[];

//----
int      draw_begin;
string   TF, IndicatorName, short_name, price_name;
datetime prevtime;
double   HP[2], Filt[3];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   if(TimeFrame <= Period()) TimeFrame = Period();
   TF = tf(TimeFrame);
   if(TF  == "Unknown") TimeFrame = Period();
   
   IndicatorDigits(Digits);
//---- indicator buffers mapping
   SetIndexBuffer(0, wave); SetIndexStyle(0,DRAW_LINE);

//---- indicator name
  
   switch(Price)
   {
   case 0 : price_name = "Close";   break;
   case 1 : price_name = "Open";    break;
   case 2 : price_name = "High";    break;
   case 3 : price_name = "Low";     break;
   case 4 : price_name = "Median";  break;
   case 5 : price_name = "Typical"; break;
   case 6 : price_name = "WClose"; break;
   }
     
   IndicatorName = WindowExpertName(); 
   short_name = IndicatorName + "[" + TF + "]("+ price_name + "," + Duration + "," + Smooth + ")";
   IndicatorShortName(short_name);
      
   SetIndexLabel(0,"Sine");
   
//----
   draw_begin = Bars - iBars(NULL,TimeFrame)*TimeFrame/Period() + Duration;
   SetIndexDrawBegin(0,draw_begin);
   
//---- initialization done
 
   return(0);
}


//----
int deinit()
{
   return(0);
}
//+------------------------------------------------------------------+
//| EvenBetterSineWave_v1 600+                                       |
//+------------------------------------------------------------------+
int start()
{
   int limit, i, shift, counted_bars = IndicatorCounted(); 
   
   
   if(counted_bars > 0) limit=Bars-counted_bars-1;
   if(counted_bars < 0) return(0);
   if(counted_bars < 1)
   { 
   limit = Bars-1;   
      for(i=limit;i>=0;i--) 
      {
      wave[i] = EMPTY_VALUE; 
      }
   }   
  
   if(TimeFrame != Period())
	{
   limit = MathMax(limit,TimeFrame/Period());   
      
      for(shift = 0;shift < limit;shift++) 
      {	
      int y = iBarShift(NULL,TimeFrame,Time[shift]);
            
      wave[shift]  = iCustom(NULL,TimeFrame,IndicatorName,0,Price,Duration,Smooth,0,y);      
      }  
	
	return(0);
	}
	else _EvenBetterSineWave(limit);

   return(0);
}

//-----

void _EvenBetterSineWave(int limit)
{   
   for(int shift=limit;shift>=0;shift--) 
   {
      if(prevtime != Time[shift])
      {
      HP[1]   = HP[0];   
      Filt[2] = Filt[1];
      Filt[1] = Filt[0];
      prevtime    = Time[shift];
      }
   
   //HighPass filter cyclic components whose periods are shorter than Duration input
   double alpha1 = (1 - MathSin(2*pi/Duration))/MathCos(2*pi/Duration);
   HP[0] = 0.5*(1 + alpha1)*(iMA(NULL,0,1,0,0,Price,shift) - iMA(NULL,0,1,0,0,Price,shift+1)) + alpha1*HP[1];
   
   //Smooth with a Super Smoother Filter from equation 3-3
   double a1 = MathExp(-1.414*pi/Smooth);
   double b1 = 2*a1*MathCos(1.414*pi/Smooth);
   double c2 = b1;
   double c3 = -a1*a1;
   double c1 = 1 - c2 - c3;
   Filt[0] = c1*(HP[0] + HP[1])/2 + c2*Filt[1] + c3*Filt[2];
   
   //3 Bar average of Wave amplitude and power
   wave[shift] = (Filt[0] + Filt[1] + Filt[2])/3;
   double Pwr  = (Filt[0]*Filt[0] + Filt[1]*Filt[1] + Filt[2]*Filt[2])/3;
   
   //Normalize the Average Wave to Square Root of the Average Power
   wave[shift] = wave[shift]/MathSqrt(Pwr);
   }
}
   

string tf(int timeframe)
{
   switch(timeframe)
   {
   case PERIOD_M1:   return("M1");
   case PERIOD_M5:   return("M5");
   case PERIOD_M15:  return("M15");
   case PERIOD_M30:  return("M30");
   case PERIOD_H1:   return("H1");
   case PERIOD_H4:   return("H4");
   case PERIOD_D1:   return("D1");
   case PERIOD_W1:   return("W1");
   case PERIOD_MN1:  return("MN1");
   default:          return("Unknown");
   }
}



